package org.itqop.chatit.commands;

import com.mojang.brigadier.CommandDispatcher;
import com.mojang.brigadier.arguments.StringArgumentType;
import net.minecraft.ChatFormatting;
import net.minecraft.commands.Commands;
import net.minecraft.commands.CommandSourceStack;
import net.minecraft.commands.arguments.EntityArgument;
import net.minecraft.commands.arguments.MessageArgument;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.entity.player.Player;
import net.minecraftforge.event.RegisterCommandsEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.event.entity.player.PlayerEvent.PlayerLoggedOutEvent;

import org.itqop.chatit.utils.PlayerConfigManager;
import org.itqop.chatit.utils.Config;
import org.itqop.chatit.utils.ProfanityMode;
import static org.itqop.chatit.utils.Utils.logPrivateMessage;

import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.Map;


@Mod.EventBusSubscriber
public class ChatITCommand {

    private static final Map<UUID, UUID> lastSenders = new ConcurrentHashMap<>();

    @SubscribeEvent
    public static void onRegisterCommands(RegisterCommandsEvent event) {
        CommandDispatcher<CommandSourceStack> dispatcher = event.getDispatcher();

        dispatcher.register(Commands.literal("chatit")
                .then(Commands.literal("adult")
                        .requires(source -> source.hasPermission(2))
                        .executes(context -> {
                            ServerPlayer player = context.getSource().getPlayerOrException();
                            PlayerConfigManager.toggleAdultContent(player);
                            boolean current = PlayerConfigManager.hasAdultContentEnabled(player);
                            String status = current ? "включен" : "выключен";
                            player.sendSystemMessage(Component.literal("Параметр adult теперь " + status + "."));
                            return 1;
                        })
                )
                .then(Commands.literal("mode")
                        .requires(source -> source.hasPermission(2))
                        .then(Commands.argument("mode", StringArgumentType.word())
                                .executes(context -> {
                                    String modeInput = StringArgumentType.getString(context, "mode").toLowerCase();
                                    ProfanityMode newMode;
                                    try {
                                        newMode = ProfanityMode.fromString(modeInput);
                                    } catch (IllegalArgumentException e) {
                                        context.getSource().sendFailure(Component.literal("Неверный режим. Используйте: off, regex, api.")
                                                .withStyle(ChatFormatting.RED));
                                        return 0;
                                    }

                                    Config.setProfanityMode(newMode);

                                    context.getSource().sendSuccess(
                                            () -> Component.literal("Режим проверки мата установлен на: " + newMode.name().toLowerCase() + ".")
                                                    .withStyle(ChatFormatting.GREEN),
                                            true
                                    );

                                    return 1;
                                })
                        )
                )
        );

        dispatcher.register(Commands.literal("r")
                .then(Commands.argument("message", MessageArgument.message())
                        .executes(context -> {
                            ServerPlayer sender = context.getSource().getPlayerOrException();
                            UUID lastSenderUUID = lastSenders.get(sender.getUUID());
                            if (lastSenderUUID == null) {
                                context.getSource().sendFailure(Component.literal("Нет игрока для ответа.")
                                        .withStyle(ChatFormatting.RED));
                                return 0;
                            }

                            ServerPlayer target = context.getSource().getServer().getPlayerList().getPlayer(lastSenderUUID);
                            if (target == null) {
                                context.getSource().sendFailure(Component.literal("Последний отправитель недоступен.")
                                        .withStyle(ChatFormatting.RED));
                                return 0;
                            }

                            Component message = MessageArgument.getMessage(context, "message");

                            return sendPrivateMessage(context.getSource(), target, message);
                        })
                )
        );
    }

    public static void registerPrivateMessageCommand(CommandDispatcher<CommandSourceStack> dispatcher, String commandName) {
        dispatcher.register(Commands.literal(commandName)
                .then(Commands.argument("target", EntityArgument.player())
                        .then(Commands.argument("message", MessageArgument.message())
                                .executes(context -> sendPrivateMessage(context.getSource(),
                                        EntityArgument.getPlayer(context, "target"),
                                        MessageArgument.getMessage(context, "message")))
                        )
                )
        );
    }

    /**
     * Метод для отправки приватного сообщения от одного игрока к другому.
     *
     * @param source  Источник команды (должен быть игроком).
     * @param target  Целевой игрок.
     * @param message Сообщение.
     * @return Статус выполнения команды.
     */
    private static int sendPrivateMessage(CommandSourceStack source, ServerPlayer target, Component message) {
        if (!(source.getEntity() instanceof ServerPlayer sender)) {
            source.sendFailure(Component.literal("Только игроки могут отправлять личные сообщения.")
                    .withStyle(ChatFormatting.RED));
            return 0;
        }

        if (sender.getUUID().equals(target.getUUID())) {
            sender.sendSystemMessage(Component.literal("Вы не можете отправлять сообщения самому себе.")
                    .withStyle(ChatFormatting.RED));
            return 0;
        }

        MutableComponent pmText = Component.literal("ЛС");
        MutableComponent fromText = Component.literal(" от ");
        MutableComponent toText = Component.literal(" для ");
        MutableComponent closeBracket = Component.literal(": ");

        MutableComponent messageFormat = message.copy().withStyle(ChatFormatting.YELLOW);

        MutableComponent messageTo = pmText.copy()
                .append(toText)
                .append(target.getDisplayName().copy().withStyle(ChatFormatting.GREEN))
                .append(closeBracket)
                .append(messageFormat);

        MutableComponent messageFrom = pmText.copy()
                .append(fromText)
                .append(sender.getDisplayName().copy().withStyle(ChatFormatting.GREEN))
                .append(closeBracket)
                .append(messageFormat);

        target.sendSystemMessage(messageFrom);
        sender.sendSystemMessage(messageTo);

        lastSenders.put(target.getUUID(), sender.getUUID());

        logPrivateMessage(sender, target, message);

        return 1;
    }

    /**
     * Слушатель события отключения игрока для очистки хэшмапы.
     *
     * @param event Событие отключения игрока.
     */
    @SubscribeEvent
    public static void onPlayerLogout(PlayerLoggedOutEvent event) {
        Player player = event.getEntity();
        UUID playerUUID = player.getUUID();

        lastSenders.values().removeIf(senderUUID -> senderUUID.equals(playerUUID));

        lastSenders.remove(playerUUID);
    }

}
